/*
 * 25.09.2004 by Andreas Grunewald
 */
package org.jd3lib;

/**
 * The picturetype Field which is used in the APIC frame.
 * 
 * <table border=0 width="70%">
 * <tr>
 * <td rowspan=21 valign=top>Picture type:</td>
 * <td>$00</td>
 * <td>Other</td>
 * </tr>
 * <tr>
 * <td>$01</td>
 * <td>32x32 pixels 'file icon' (PNG only)</td>
 * </tr>
 * 
 * <tr>
 * <td>$02</td>
 * <td>Other file icon</td>
 * </tr>
 * <tr>
 * <td>$03</td>
 * <td>Cover (front)</td>
 * </tr>
 * <tr>
 * <td>$04</td>
 * <td>Cover (back)</td>
 * </tr>
 * <tr>
 * <td>$05</td>
 * <td>Leaflet page</td>
 * </tr>
 * <tr>
 * <td>$06</td>
 * <td>Media (e.g. lable side of CD)</td>
 * </tr>
 * <tr>
 * <td>$07</td>
 * <td>Lead artist/lead performer/soloist</td>
 * </tr>
 * 
 * <tr>
 * <td>$08</td>
 * <td>Artist/performer</td>
 * </tr>
 * <tr>
 * <td>$09</td>
 * <td>Conductor</td>
 * </tr>
 * <tr>
 * <td>$0A</td>
 * <td>Band/Orchestra</td>
 * </tr>
 * <tr>
 * <td>$0B</td>
 * <td>Composer</td>
 * </tr>
 * <tr>
 * <td>$0C</td>
 * <td>Lyricist/text writer</td>
 * </tr>
 * <tr>
 * <td>$0D</td>
 * <td>Recording Location</td>
 * </tr>
 * 
 * <tr>
 * <td>$0E</td>
 * <td>During recording</td>
 * </tr>
 * <tr>
 * <td>$0F</td>
 * <td>During performance</td>
 * </tr>
 * <tr>
 * <td>$10</td>
 * <td>Movie/video screen capture</td>
 * </tr>
 * <tr>
 * <td>$11</td>
 * <td>A bright coloured fish</td>
 * </tr>
 * <tr>
 * <td>$12</td>
 * <td>Illustration</td>
 * </tr>
 * <tr>
 * <td>$13</td>
 * <td>Band/artist logotype</td>
 * </tr>
 * 
 * <tr>
 * <td>$14</td>
 * <td>Publisher/Studio logotype</td>
 * </tr>
 * </table>
 * 
 * @author Andreas Grunewald
 *  
 */
public class FieldPictureType
        extends Id3Field {

    /**
     * $00 Other
     */
    public static final byte OTHER = 0x00;

    /**
     * $01 32x32 pixels 'file icon' (PNG only) <br />
     */
    public static final byte ICON32X32 = 0x01;

    /**
     * $02 Other file icon <br />
     */
    public static final byte OTHER_ICON = 0x02;

    /**
     * $03 Cover (front) <br />
     */
    public static final byte COVER_FRONT = 0x03;

    /**
     * $04 Cover (back) <br />
     */
    public static final byte COVER_BACK = 0x04;

    /**
     * $05 Leaflet page <br />
     */
    public static final byte LEAFLET_PAGE = 0x05;

    /**
     * $06 Media (e.g. lable side of CD) <br />
     */
    public static final byte MEDIA = 0x06;

    /**
     * $07 Lead artist/lead performer/soloist <br />
     */
    public static final byte LEAD_ARTIST = 0x07;

    /**
     * $08 Artist/performer <br />
     */
    public static final byte ARTIST = 0x08;

    /**
     * $09 Conductor <br />
     */
    public static final byte CONDUCTOR = 0x09;

    /**
     * $0A Band/Orchestra <br />
     */
    public static final byte BAND = 0x0A;

    /**
     * $0B Composer <br />
     */
    public static final byte COMPOSER = 0x0B;

    /**
     * $0C Lyricist/text writer <br />
     */
    public static final byte LYRICIST = 0x0C;

    /**
     * $0D Recording Location <br />
     */
    public static final byte RECORDING_LOCATION = 0x0D;

    /**
     * $0E During recording <br />
     */
    public static final byte DURING_RECORDING = 0x0E;

    /**
     * $0F During performance <br />
     */
    public static final byte DURING_PERFORMANCE = 0x0F;

    /**
     * $10 Movie/video screen capture <br />
     */
    public static final byte VIDEO_CAPTURE = 0x10;

    /**
     * $11 A bright coloured fish <br />
     */
    public static final byte A_BRIGHT_COLOURED_FISH = 0x11;

    /**
     * $12 Illustration <br />
     */
    public static final byte ILLUSTRATION = 0x12;

    /**
     * $13 Band/artist logotype <br />
     */
    public static final byte BAND_LOGOTYPE = 0x13;

    /**
     * $14 Publisher/Studio logotype <br />
     *  
     */
    public static final byte STUDIO_LOGOTYPE = 0x14;

    /**
     * The Type of the Picture
     */
    private byte type;

    /**
     * should be the smallest of the public constants
     */
    private static final byte MIN = OTHER;

    /**
     * should be the biggest of the public constants
     */
    private static final byte MAX = STUDIO_LOGOTYPE;

    /**
     * @param i
     */
    public FieldPictureType(byte input) {
        this.setField(input);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.jd3lib.Id3Field#getBytes()
     */
    public byte[] getBytes() {
        return new byte[] { type };
    }

    public void setField(Integer inType) {
        this.setField((inType.byteValue()));
    }

    public void setField(Byte inType) {
        this.setField(inType.byteValue());
    }

    public void setField(byte[] inType) {
        this.setField(inType[0]);
    }

    public void setField(int[] inType) {
        this.setField((byte) (inType)[0]);
    }

    /**
     * @param b
     */
    public void setField(byte inType) {
        if (inType >= FieldPictureType.MIN && inType <= FieldPictureType.MAX) {
            this.type = inType;
        } else
            throw new IllegalArgumentException("Only bytes in the Range from "
                    + FieldPictureType.MIN + " to " + FieldPictureType.MAX
                    + " are allowed");
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.jd3lib.Id3Field#getValue()
     */
    public Object getValue() {
        return new Byte(this.type);
    }

}