/*
 * $Id: Id3FrameAPIC.java,v 1.10 2005/04/27 08:36:43 gruni Exp $
 * $Author: gruni $ 
 * 
 * $Log: Id3FrameAPIC.java,v $
 * Revision 1.10  2005/04/27 08:36:43  gruni
 * changes due to refactoring
 *
 */
package org.jd3lib;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.ByteBuffer;

/**
 * 4.15. Attached picture
 * 
 * This frame contains a picture directly related to the audio file. Image
 * format is the MIME type and subtype [MIME] for the image. In the event that
 * the MIME media type name is omitted, &quot;image/&quot; will be implied. The
 * &quot;image/png&quot; [PNG] or &quot;image/jpeg&quot; [JFIF] picture format
 * should be used when interoperability is wanted. Description is a short
 * description of the picture, represented as a terminated textstring. The
 * description has a maximum length of 64 characters, but may be empty. There
 * may be several pictures attached to one file, each in their individual
 * &quot;APIC&quot; frame, but only one with the same content descriptor. There
 * may only be one picture with the picture type declared as picture type $01
 * and $02 respectively. There is the possibility to put only a link to the
 * image file by using the 'MIME type' &quot;--&lt;&quot; and having a complete
 * URL [URL] instead of picture data. The use of linked files should however be
 * used sparingly since there is the risk of separation of files.
 * 
 * &lt;Header for 'Attached picture', ID: &quot;APIC&quot;&gt;
 * 
 * Text encoding $xx
 * 
 * MIME type &lt;text string&gt; $00
 * 
 * Picture type $xx
 * 
 * Description &lt;text string according to encoding&gt; $00 (00)
 * 
 * Picture data &lt;binary data&gt;
 * 
 * 
 * Picture type:
 * 
 * $00 Other <br />
 * 
 * $01 32x32 pixels 'file icon' (PNG only) <br />
 * 
 * $02 Other file icon <br />
 * 
 * $03 Cover (front) <br />
 * 
 * $04 Cover (back) <br />
 * 
 * $05 Leaflet page <br />
 * 
 * $06 Media (e.g. lable side of CD) <br />
 * 
 * $07 Lead artist/lead performer/soloist <br />
 * 
 * $08 Artist/performer <br />
 * 
 * $09 Conductor <br />
 * 
 * $0A Band/Orchestra <br />
 * 
 * $0B Composer <br />
 * 
 * $0C Lyricist/text writer <br />
 * 
 * $0D Recording Location <br />
 * 
 * $0E During recording <br />
 * 
 * $0F During performance <br />
 * 
 * $10 Movie/video screen capture <br />
 * 
 * $11 A bright coloured fish <br />
 * 
 * $12 Illustration <br />
 * 
 * $13 Band/artist logotype <br />
 * 
 * $14 Publisher/Studio logotype <br />
 * 
 * @author Andreas Grunewald
 *  
 */
public class Id3FrameAPIC
    extends Id3FrameMultiple {

  /**
   * @param frameData
   * @param header
   */
  public Id3FrameAPIC(ByteBuffer frameData, Id3v2FrameHeader header) {
    super(frameData, header);
    //LATER 0.7 Auto-generated constructor stub
  }

  FieldTerminatedString description;//<text string according to encoding> $00

  FieldTerminatedString mimeType;//<text string> $00

  // (00)

  FieldPictureData pictureData;//<binary data>

  FieldPictureType pictureType;//$xx

  FieldEncoding textEncoding;// $xx

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3Frame#getDataString()
   */
  protected String getFrameString() {
    // LATER 0.1 Auto generated method stub
    return null;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3Frame#initialize(java.io.InputStream,
   *      org.jd3lib.Id3v2FrameHeader)
   */
  protected void initializeExisting(InputStream tagData,
      Id3v2FrameHeader header2) {
    System.out.println("RunningAPIC initialize");
    byte[] temp = new byte[header2.getSize()];
    try {
      tagData.read(temp);
    } catch (IOException e) {
      //LATER 1.5 Auto-generated catch block
      e.printStackTrace();
    }
    ByteArrayInputStream inStream = new ByteArrayInputStream(temp);
    textEncoding = new FieldEncoding((byte) inStream.read());
    // System.out.println("["+textEncoding+"]");
    mimeType = new FieldTerminatedString(FieldTerminatedString
        .readUntilZero(inStream));
    // System.out.println("[" + mimeType + "]");
    pictureType = new FieldPictureType((byte) inStream.read());
    // System.out.println("[" + pictureType + "]");
    description = new FieldTerminatedString(FieldTerminatedString
        .readUntilZero(inStream));
    // System.out.println("["+description+"]");

    pictureData = new FieldPictureData(FieldPictureData.processBytes(inStream,
        inStream.available()));
    //pictureData = new
    // FieldPictureData(processUntilEnd(inStream).toByteArray());
    System.out.println("Now are " + inStream.available() + " bytes available");

  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3Frame#initializeExisting(java.nio.ByteBuffer,
   *      org.jd3lib.Id3v2FrameHeader)
   */
  protected void initializeExisting(ByteBuffer frameData,
      Id3v2FrameHeader header) {
    // DONEXT rework to ByteBuffer
    // System.out.println("RunningAPIC initialize");
    ByteArrayInputStream inStream = new ByteArrayInputStream(frameData.array());
    textEncoding = new FieldEncoding((byte) inStream.read());
    // System.out.println("["+textEncoding+"]");
    mimeType = new FieldTerminatedString(FieldTerminatedString
        .readUntilZero(inStream));
    // System.out.println("[" + mimeType + "]");
    pictureType = new FieldPictureType((byte) inStream.read());
    // System.out.println("[" + pictureType + "]");
    description = new FieldTerminatedString(FieldTerminatedString
        .readUntilZero(inStream));
    // System.out.println("["+description+"]");

    pictureData = new FieldPictureData(FieldPictureData.processBytes(inStream,
        inStream.available()));
    //pictureData = new
    // FieldPictureData(processUntilEnd(inStream).toByteArray());
    // System.out.println("Now are " + inStream.available()
    //                + " bytes available");

  }

  public void writePicture(OutputStream out) {
    try {
      out.write(pictureData.getBytes());
    } catch (IOException e) {
      //LATER 1.5 Auto-generated catch block
      e.printStackTrace();
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3Frame#initializeNew(org.jd3lib.Id3v2FrameHeader)
   */
  protected void initializeNew(Id3v2FrameHeader header) {
    //TODO init the variables
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3Frame#getFrameData()
   */
  protected ByteArrayOutputStream getFrameData() {
    //TODO Implement real get FrameData
    return new ByteArrayOutputStream();
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3FrameMultiple#isDifferent(org.jd3lib.Id3FrameMultiple)
   */
  public boolean isDifferent(Id3FrameMultiple toCompare) {
    // LATER 0.1 Auto generated method stub
    return false;
  }

}