/*
 * 22.08.2004 by Andreas Grunewald
 */
package org.jd3lib;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * @author Andreas Grunewald
 * 
 * LATER 1.0 Write Documentation
 */
public class Id3FrameList {

    Map list;

    Map listFrames;

    public Id3FrameList() {
        list = new LinkedHashMap(8);
    }

    public boolean containsFrame(String frameID) {
        return list.containsKey(frameID);
    }

    /**
     * Returns the Frame or the first Frame if more than one Frame of a kind are
     * available
     * 
     * @param frameID
     * @return
     */
    public Id3Frame getFrame(String frameID) {
        Object out = list.get(frameID);
        if (out != null && out instanceof FrameContainer) {
            return ((FrameContainer) out).get(0);
        }
        return (Id3Frame) out;
    }

    /**
     * returns how often a frame is in the tag.
     */
    public int getFrameCount(String frameID) {
        if (containsFrame(frameID)) {
            Object fr = list.get(frameID);
            if (fr instanceof FrameContainer) {
                return ((FrameContainer) fr).size();
            }
            return 1;
        }
        return 0;
    }

    /**
     * @return
     */
    public ByteArrayOutputStream getFramesData() {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        //Enumeration enum = list.elements();

        Iterator iter = list.values().iterator();
        while (iter.hasNext()) {
            Object temp = iter.next();
            if (temp instanceof Id3Frame) {
                try {
                    ((Id3Frame) temp).getAllData().writeTo(out);
                } catch (IOException e) {
                    e.printStackTrace();
                }
            } else if (temp instanceof FrameContainer) {
                System.out.println("ERROR unimplemented");
                //FIXME Implement with following
                //((FrameContainer)temp).getAllData().writeTo(out);
                //	throw new RuntimeException(
                //		"This case should never occure please report this problem to
                // the developer");
            } else {
                //System.out.println("#########\n
                // temp:"+temp.getClass().getName()+"\n###########");
                //TODO check if else is Possible

                throw new RuntimeException(
                        "This case should never occure please report this problem to the developer");

            }
        }
        /*
         * while (enum.hasMoreElements()) { Object temp = enum.nextElement(); if
         * (temp instanceof Id3Frame) { try { ((Id3Frame)
         * temp).getAllData().writeTo(out); } catch (IOException e) {
         * e.printStackTrace(); } } else if (temp instanceof FrameContainer) {
         * //FIXME Implement with following
         * //((FrameContainer)temp).getAllData().writeTo(out); throw new
         * RuntimeException( "This case should never occure please report this
         * problem to the developer"); } else { //TODO check if else is Possible
         * throw new RuntimeException( "This case should never occure please
         * report this problem to the developer"); } }
         */
        return out;
    }

    public Id3Frame getMultiFrame(String frameID, int number) {
        Id3Frame out = null;
        if (containsFrame(frameID)) {
            if (number == 0)
                out = getFrame(frameID);
            else {
                Object fr = list.get(frameID);
                if (fr instanceof FrameContainer
                        && ((FrameContainer) fr).getType().equals(frameID)
                        && number < ((FrameContainer) fr).size()) {
                    out = ((FrameContainer) fr).get(number);
                }
            }
        }
        return out;
    }

    public void putFrame(Id3Frame theFrame) {
        //TODO Try to check what happens with multiple occurances of Frames
        // with
        // same ID
        if (theFrame instanceof Id3FrameMultiple
                && list.containsKey(theFrame.getFrameID())) {
            Object fr = list.get(theFrame.getFrameID());
            // System.out.println(fr.getClass().getName());
            if (fr instanceof FrameContainer
                    && ((FrameContainer) fr).getType().equals(
                            theFrame.getFrameID())) {
                ((FrameContainer) fr).add((Id3FrameMultiple) theFrame);
            } else {
                Id3FrameMultiple multFrame = (Id3FrameMultiple) fr;
                FrameContainer cont = new FrameContainer(multFrame);
                cont.add((Id3FrameMultiple) theFrame);
                list.put(cont.getType(), cont);
            }
            //TODO make sure the two frames are different
            /*
             * if (!(check.equals(theFrame))) { } else throw new
             * IllegalArgumentException( "Adding this frame is not allowed");
             */
        } else
            list.put(theFrame.getFrameID(), theFrame);
    }

    public void removeFrame(String frameID) {
        Object out = list.get(frameID);
        if (out != null && out instanceof FrameContainer)
            ((FrameContainer) out).remove(0);
        else
            list.remove(frameID);
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */

    public synchronized String toString() {
        //Enumeration enum = list.elements();
        Iterator iter = list.values().iterator();
        String out = "";
        while (iter.hasNext()) {
            out += iter.next().toString() + "\n\n";
        }
        //while (enum.hasMoreElements()) {
        //  out += enum.nextElement().toString() + "\n\n";
        //}
        return out;
    }

    /**
     * @return
     */
    public byte[] writeBytes() {
        // LATER 0.1 Auto generated method stub
        return null;
    }

}