/*
 * $Id: Id3FrameUFID.java,v 1.8 2005/04/27 08:44:16 gruni Exp $
 * $Author: gruni $ 
 * 
 * $Log: Id3FrameUFID.java,v $
 * Revision 1.8  2005/04/27 08:44:16  gruni
 * changes due to refactoring
 *
 */
package org.jd3lib;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.net.URI;
import java.nio.ByteBuffer;

/**
 * 4.1. Unique file identifier
 * 
 * This frame's purpose is to be able to identify the audio file in a database
 * that may contain more information relevant to the content. Since
 * standardisation of such a database is beyond this document, all frames begin
 * with a null-terminated string with a URL [URL] containing an email address,
 * or a link to a location where an email address can be found, that belongs to
 * the organisation responsible for this specific database implementation.
 * Questions regarding the database should be sent to the indicated email
 * address. The URL should not be used for the actual database queries. The
 * string "http://www.id3.org/dummy/ufid.html" should be used for tests.
 * Software that isn't told otherwise may safely remove such frames. The 'Owner
 * identifier' must be non-empty (more than just a termination). The 'Owner
 * identifier' is then followed by the actual identifier, which may be up to 64
 * bytes. There may be more than one "UFID" frame in a tag, but only one with
 * the same 'Owner identifier'.
 * 
 * <Header for 'Unique file identifier', ID: "UFID"> Owner identifier <text
 * string> $00 Identifier <up to 64 bytes binary data>
 * 
 * @author Andreas Grunewald FIXME implement methods
 *  
 */
public class Id3FrameUFID
    extends Id3FrameMultiple {

  /**
   * @param frameData
   * @param header
   */
  public Id3FrameUFID(ByteBuffer frameData, Id3v2FrameHeader header) {
    super(frameData, header);
    // LATER 0.7 Auto-generated constructor stub
  }

  FieldUFIDData Identifier;

  FieldTerminatedString OwnerIdentifier;

  /**
   * @return Returns the identifier.
   */
  public FieldUFIDData getIdentifier() {
    return Identifier;
  }

  /**
   * @return Returns the ownerIdentifier.
   */
  public String getOwnerIdentifier() {
    return (String) OwnerIdentifier.getValue();
  }

  /**
   * @param identifier
   *          The identifier to set.
   */
  public void setIdentifier(byte[] id) {
    Identifier = new FieldUFIDData(id);
  }

  //    public Id3FrameUFID(URI Owner,byte[] ident){
  //       OwnerIdentifier = new FieldTerminatedString(Owner.toString());
  //       Identifier = new FieldBinaryData(ident);
  //    }

  public void setOwnerIdentifier(URI Owner) {
    OwnerIdentifier = new FieldTerminatedString(Owner.toString());
  }

  /*
   * (non-Javadoc)
   * 
   * @see de.gruni.thejee.Id3Frame#initialize(java.io.InputStream,
   *      de.gruni.thejee.Id3v2FrameHeader)
   */
  protected void initializeExisting(InputStream tagData,
      Id3v2FrameHeader header2) {
    // LATER 0.1 Auto generated method stub

  }

  /*
   * (non-Javadoc)
   * 
   * @see de.gruni.thejee.Id3Frame#getDataString()
   */
  protected String getFrameString() {
    // LATER 0.1 Auto generated method stub
    return null;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3Frame#initializeNew(org.jd3lib.Id3v2FrameHeader)
   */
  protected void initializeNew(Id3v2FrameHeader header) {
    // LATER 0.1 Auto generated method stub

  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3Frame#getFrameData()
   */
  protected ByteArrayOutputStream getFrameData() {
    // LATER 0.1 Auto generated method stub
    return null;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3FrameMultiple#isDifferent(org.jd3lib.Id3FrameMultiple)
   */
  public boolean isDifferent(Id3FrameMultiple toCompare) {
    // LATER 0.1 Auto generated method stub
    return false;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.Id3Frame#initializeExisting(java.nio.ByteBuffer,
   *      org.jd3lib.Id3v2FrameHeader)
   */
  protected void initializeExisting(ByteBuffer frameData,
      Id3v2FrameHeader header) {
    // LATER 0.1 Auto generated method stub

  }

}