/*
 * 29.08.2004 by Andreas Grunewald
 */
package org.jd3lib;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;

import org.jd3lib.util.Coder;

/**
 * Frame ID $xx xx xx xx (four characters) Size $xx xx xx xx Flags $xx xx
 * 
 * @author Andreas Grunewald
 */
public class Id3v2FrameHeader {

  private String frameID;

  private int size;

  protected Id3v2FrameHeader() {
    frameID = "";
    size = -1;
  }

  public Id3v2FrameHeader(String id) {
    frameID = id;
    size = -1;
    //When Constructing a new Frame the defautFlags have to be retrieved
    //Flags = new byte[] { 0x00, 0x00 };
  }

  /**
   * @param frameID
   * @deprecated The BAseclass should not have these values
   */
  public Id3v2FrameHeader(InputStream tagData) {
    super();
    byte[] header = new byte[10];
    try {
      tagData.read(header);
    } catch (IOException e1) {
      //LATER 1.5 Auto-generated catch block
      e1.printStackTrace();
    }
    if (header[0] == 0) {
      readoutPadding(tagData);
    } else {
      frameID = "" + (char) header[0] + (char) header[1] + (char) header[2]
          + (char) header[3];
      byte[] temp = { header[4], header[5], header[6], header[7] };

      //for ID3v2.4 the Framesize is syncsafe
      //Size = Coder.getSyncsafeIntegerValue(temp);

      //if ID3v2.3 the Framesize is not syncsafe
      size = Coder.getIntegerValue(temp);

      // Flags[0] = header[8];
      // Flags[1] = header[9];
    }

  }

  /**
   * @param headbuf
   */
  public Id3v2FrameHeader(ByteBuffer headbuf) {
    super();
    headbuf.rewind();
    byte[] header = headbuf.array();
    if (header[0] == 0) {
      readoutPadding(headbuf);
    } else {
      frameID = "" + (char) header[0] + (char) header[1] + (char) header[2]
          + (char) header[3];
      byte[] temp = { header[4], header[5], header[6], header[7] };

      //for ID3v2.4 the Framesize is syncsafe
      //Size = Coder.getSyncsafeIntegerValue(temp);

      //if ID3v2.3 the Framesize is not syncsafe
      size = Coder.getIntegerValue(temp);

      //  Flags[0] = header[8];
      // Flags[1] = header[9];
    }
  }

  /**
   * @param headbuf
   */
  protected void readoutPadding(ByteBuffer headbuf) {
    frameID = "ZPAD";
    size = headbuf.limit();
  }

  /**
   * @return Returns the flags.
   * @deprecated there are no FLags in V2_2
   */
  public byte[] getFlags() {
    return null;
  }

  /**
   * @return Returns the frameID.
   */
  public String getFrameID() {
    return frameID;
  }

  /**
   * @return Returns the size.
   */
  public int getSize() {
    return size;
  }

  /**
   * @param tagData
   */
  private void readoutPadding(InputStream tagData) {
    frameID = "ZPAD";
    size = 10;
    byte[] current = new byte[1];
    try {
      while (current[0] != (byte) 0xFF) {
        size++;
        tagData.read(current);
      }
    } catch (IOException e) {
      //LATER 1.5 Auto-generated catch block
      e.printStackTrace();
    }
  }

  /**
   * @param flags
   *          The flags to set.
   * @deprecated no flags in basic version
   */
  public void setFlags(byte[] flags) {
    // Flags = flags;
  }

  /**
   * @param frameID
   *          The frameID to set.
   */
  public void setFrameID(String inFrameID) {
    frameID = inFrameID;
  }

  /**
   * @param size
   *          The size to set.
   */
  public void setSize(int inSize) {
    size = inSize;
  }

  public String toString() {
    return "Frame ID " + getFrameID() + "\n" + "Size     " + getSize() + "\n";

  }

  /**
   * @return
   * @deprecated doesn't work anymore because of flags
   */
  public ByteArrayOutputStream getRawData() {
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    try {
      out.write(frameID.getBytes());
      out.write(Coder.getBytes(size));
      //        out.write(Flags);
    } catch (IOException e) {
      //LATER 1.5 Auto-generated catch block
      e.printStackTrace();
    }

    return out;
  }
}