/*
 * $Id: MP3File.java,v 1.7 2005/04/27 09:05:59 gruni Exp $
 * $Author: gruni $ 
 * 
 * $Log: MP3File.java,v $
 * Revision 1.7  2005/04/27 09:05:59  gruni
 * minor changes due to refactoring
 *
 */
package org.jd3lib;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;

/**
 * @author Andreas Grunewald
 * 
 * LATER 1.0 Write Documentation
 */
public class MP3File implements AudioFile {

  private final File theOriginal;

  // private String filePath;

  private long MP3beginning;

  private MPEGInfo mpegI;

  private Id3v1Tag v1tag;

  private Id3v2Tag v2tag;

  private boolean iD3v2parsed;

  private boolean iD3v1parsed;

  /**
   * Associates this Object with a file
   * 
   * @param string
   *          the filePath
   */
  public MP3File(String string) {
    this(new File(string));
  }

  /**
   * @param currentFile
   */
  public MP3File(File currentFile) {
    theOriginal = currentFile;
    try {

      //this.filePath = theOriginal.getAbsolutePath();
      FileChannel theStream = new FileInputStream(theOriginal).getChannel();

      MP3beginning = theStream.size();
      //TODO remove System.out.println("Before Id3v2:" +
      // theStream.available());
      //this.v2tag = new Id3v2Tag(theStream);
      //MP3beginning -= theStream.available() + 1;
      //TODO remove System.out.println("Before MPEG:" +
      // theStream.available());
      //this.mpegI = new MPEGInfo(theStream);
      //TODO remove System.out.println("Before v1:" +
      // theStream.available());
      //this.v1tag = new Id3v1Tag(theStream);
    } catch (FileNotFoundException e) {
      // LATER 1.5 Auto-generated catch block
      e.printStackTrace();
    } catch (IOException e) {
      // LATER 1.5 Auto-generated catch block
      e.printStackTrace();
    }
  }

  /**
   * @return
   */
  private byte[] getMP3() {

    try {
      FileInputStream temp = new FileInputStream(getFilePath());
      FileChannel tChan = temp.getChannel();
      ByteBuffer byBu = ByteBuffer
          .allocate((int) ((int) tChan.size() - MP3beginning));
      tChan.read(byBu, MP3beginning);
      return byBu.array();
    } catch (FileNotFoundException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    }
    return new byte[] {};
  }

  /**
   * @return
   */
  public byte[] getData() {
    ByteArrayOutputStream tag = new ByteArrayOutputStream();
    if (this.hasID3v2()) try {
      this.getID3v2().getTagData().writeTo(tag);
      tag.write(this.getMP3());
      // if (this.hasID3v1())
        // tag.write(this.getID3v1().getTagData().toByteArray());
      } catch (IOException e) {
        // LATER 1.5 Auto-generated catch block
        e.printStackTrace();
      }
    return tag.toByteArray();
  }

  /**
   * Returns the canonical path to this file or /tmpNonExistingFile if the file
   * doesn't exist. TODO refactor is method see if it makes sense to throw an
   * exception and which type of exception.
   */
  public String getFilePath() {
    try {
      return theOriginal.getCanonicalPath();
    } catch (IOException e) {
      return "/tmpNonExistingFile";
    }
  }

  /**
   * @return
   */
  public Id3v1Tag getID3v1() {
    if (v1tag == null && iD3v1parsed == false) {
      FileChannel chan;
      try {
        chan = new FileInputStream(theOriginal).getChannel();
        //FIXME v1.0 Workaround Bugfix for Files with ZERO length
        if (chan.size() < 128) return null;
        chan.position(chan.size() - 128);
        ByteBuffer mybyte = ByteBuffer.allocate(128);
        chan.read(mybyte);
        mybyte.flip();
        try {
          v1tag = new Id3v1Tag(mybyte);
        } catch (InstantiationException e) {
          //Just don't do anything if there is no Tag then the
          //Tag is null
        }

      } catch (FileNotFoundException e) {
        // LATER 1.5 Auto-generated catch block
        e.printStackTrace();
      } catch (IOException e) {
        // LATER 1.5 Auto-generated catch block
        e.printStackTrace();
      }

    }
    return v1tag;
  }

  /**
   * @return
   */
  public Id3v2Tag getID3v2() {
    if (v2tag == null && iD3v2parsed == false) {
      iD3v2parsed = true;
      FileChannel theStream;
      try {
        theStream = new FileInputStream(theOriginal).getChannel();
        MP3beginning = theStream.size();
        try {
          v2tag = new Id3v2Tag(theStream);
        } catch (InstantiationException e) {
          //TODO check if Throwing or printing stacktrace
        }
        //LATER remove System.out.println("Before Id3v2:" +
        // theStream.available());
        //this.v2tag = new Id3v2Tag(theStream);
      } catch (FileNotFoundException e) {
        //TODO v4.0 Auto-generated catch block
        e.printStackTrace();
      } catch (IOException e1) {
        //TODO v4.0 Auto-generated catch block
        e1.printStackTrace();
      }
    }
    return v2tag;
  }

  /**
   * @return
   */
  public MPEGInfo getMEPGInfo() {
    // LATER 0.1 Auto generated method stub
    return null;
  }

  /**
   * Tests if the file has an ID3v1 tag if the tag hasn't been parsed it will
   * first try to parse the tag before returning a value
   * 
   * @return
   */
  public boolean hasID3v1() {
    if (iD3v1parsed == false) this.getID3v1();
    return v1tag != null;
  }

  /**
   * Tests if the file has an ID3v2 tag if the tag hasn't been parsed it will
   * first try to parse the tag before returning a value
   * 
   * @return
   */
  public boolean hasID3v2() {
    if (iD3v2parsed == false) this.getID3v2();
    return v2tag != null;
  }

  /**
   * @return
   */
  public boolean hasMPEGInfo() {
    return mpegI != null;
  }

  /**
   *  
   */
  public void stripTags() {
    v2tag = null;
    v1tag = null;
  }

  /**
   * @param string
   *  
   */
  public void writeToDisk(String string) {
    String newFileName = this.getFilePath();
    if (string != null)
        newFileName += string + System.currentTimeMillis() + ".mp3";
    try {
      FileOutputStream tag = new FileOutputStream(newFileName);
      tag.write(getData());
      tag.close();
    } catch (Exception e) {
      // LATER 1.5 Auto-generated catch block
      e.printStackTrace();
    }

  }

  public File getTheFile() {
    return theOriginal;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.AudioFile#getMetaData()
   */
  public MetaData getMetaData() {
    //LATER check if correct or if better option
    if (this.hasID3v2()) return getID3v2();
    return getID3v1();
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.AudioFile#getFileName()
   */
  public String getFileName() {
    //TODO 2.0 compare if this is more robust
    // currentFile.getName().split("\\Q.\\E...")[0]
    return theOriginal.getName().substring(0,
        theOriginal.getName().lastIndexOf('.'));
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.jd3lib.AudioFile#getFileType()
   */
  public String getFileType() {
    return theOriginal.getName().substring(
        theOriginal.getName().lastIndexOf('.'), theOriginal.getName().length());
  }
}