/*
 * $Id: MetaData.java,v 1.1 2005/01/27 17:11:07 gruni Exp $
 * $Author: gruni $ 
 * $Revision: 1.1 $
 * $Date: 2005/01/27 17:11:07 $
 * $Log: MetaData.java,v $
 * Revision 1.1  2005/01/27 17:11:07  gruni
 * New Metadata classes
 *
 * Revision 1.8  2004/12/16 16:59:52  gruni
 * a lot of updates switching to NIO and adding ID3v1
 */
package org.jd3lib;

/**
 * This class implements a General metaData interface, which needs to be
 * implemented when a class wants to provide access to the general metaData
 * Classes can support other more specific access methods but this is the common
 * way to access the smallest common Fields.
 * 
 * @author Grunewald
 * @version 0.9
 */
public interface MetaData {

    /**
     * Return the title field of the tag
     * 
     * @return the title field of the tag
     */
    public abstract String getTitle();

    /**
     * Set the title field of the tag. The maximum size of the String is 30. If
     * the size exceeds the maximum size, the String will be truncated.
     * 
     * @param newTitle
     *            the title for the tag
     */
    public abstract void setTitle(String newTitle);

    /**
     * Return the artist field of the tag
     * 
     * @return the artist field of the tag
     */
    public abstract String getArtist();

    /**
     * Set the artist field of the tag. The maximum size of the String is 30. If
     * the size exceeds the maximum size, the String will be truncated.
     * 
     * @param newArtist
     *            the artist for the tag
     */
    public abstract void setArtist(String newArtist);

    /**
     * Return the album field of the tag
     * 
     * @return the album field of the tag
     */
    public abstract String getAlbum();

    /**
     * Set the album field of the tag. The maximum size of the String is 30. If
     * the size exceeds the maximum size, the String will be truncated.
     * 
     * @param newAlbum
     *            the album for the tag
     */
    public abstract void setAlbum(String newAlbum);

    /**
     * Return the year field of the tag
     * 
     * @return the year field of the tag
     */
    public abstract int getYear();

    /**
     * Set the year field of the tag. The maximum size of the String is 4. If
     * the size exceeds the maximum size, the String will be truncated.
     * 
     * @param newYear
     *            the year for the tag
     */
    public abstract void setYear(int newYear);

    /**
     * Return the comment field of the tag
     * 
     * @return the comment field of the tag
     */
    public abstract String getComment();

    /**
     * Set the comment field of the tag. The maximum size of the String is 30.
     * If the size exceeds the maximum size, the String will be truncated.
     * 
     * @param newComment
     *            the comment of the tag
     */
    public abstract void setComment(String newComment);

    /**
     * Return the track field of the tag, or -1 if the tag does not exist.
     * 
     * @return the track field of the tag
     */
    public abstract int getTrack();

    /**
     * Set the track field of the tag. The track number has to be between 0 and
     * 255. If it is not, nothing will happen.
     * 
     * @param newTrack
     *            the track of the tag
     */
    public abstract void setTrack(int newTrack);

    /**
     * Return the genre field of the tag, -1 if tag doesn't exist
     * 
     * @return the genre field of the tag
     */
    public abstract String getGenre();

    /**
     * Set the genre field of the tag. This probably should not be greater than
     * 115, but supports values from 0-255.
     * 
     * @param newGenre
     *            the genre of the tag
     * @exception ID3FieldDataException
     *                if the value supplie is invalid
     */
    public abstract void setGenre(String newGenre);

    /**
     * Returns a String representation of this object. Contains all information
     * within the tag.
     * 
     * @return a String representation of this object
     */
    public abstract String toString();
}