/*
 * Created on 01.11.2004
 */
package org.jd3lib.archoslib;

import java.nio.ByteBuffer;
import java.util.Vector;

import org.jd3lib.archoslib.lists.List;
import org.jd3lib.archoslib.lists.ListAsRoot;
import org.jd3lib.archoslib.lists.ListOfAlbums;
import org.jd3lib.archoslib.lists.ListOfArtists;
import org.jd3lib.archoslib.lists.ListOfSong;

/**
 * <p>3 - Lists with num_list items.</p>
 * 
 * <pre><code>
 struct list_str { 
 uint8_t       type;     list type 
 uint24_t      offset;   pointer to list entries 
 uint16_t      length;   number of list entries 
 file_list_t   parent;   pointer to parent list 
 string_t      name;     name of list 
 } </code></pre>
 * 
 * <p>Currently these values for type are defined:</p> <table width="60%">
 * <tr><td width="5%">0</td><td width="20%">LIST_ROOT</td>
 * <td width="30%">(root list is always first list in library)</td></tr><tr>
 * <td>1</td><td>LIST_ARTIST</td><td rowspan="6">&nbsp;</td></tr><tr>
 * <td>2</td><td>LIST_ALBUM</td></tr><tr><td>3</td><td>LIST_SONG
 * </td></tr><tr><td>4</td><td>LIST_M3U</td></tr><tr><td>5</td>
 * <td>LIST_GENRE</td></tr><tr><td>6</td><td>LIST_YEAR</td></tr>
 * </table> <p>The list type is used by the player SW GUI to show different
 * "icons" for different types of lists</p><p>If the list is the "search
 * list" as indicated in the file header and the list is "hidden" (not
 * accessible from list structure) then the root list should be given as parent.
 * </p>
 * 
 * @author Grunewald
 * @version 0.9
 *  
 */
public class Lists extends Section implements ArcFileProcessor {

  /**
   * Vector&lt;List&gt; containing all Lists either ListOfList or ListOfFiles
   */
  private Vector NGlists;

  /**
   * Root list reference for better access.
   */
  private ListAsRoot root;

  public Lists(Strings string) {
    root = new ListAsRoot(Strings.ROOT, string);
    configureRoot();
  }

  /**
   *  
   */
  private void configureRoot() {
    root.configure(new ListOfArtists(Strings.ARTISTS, root));
    root.configure(new ListOfAlbums(Strings.ALBUMS, root));
    root.configure(new ListOfSong(Strings.SONGS, ListOfSong.SORT_BY_TITLE, root));
  }

  /**
   * creates output for Lists part of the ARCLib
   *  
   */
  protected ByteBuffer toByteBuffer() {
    //root.close(ents);
    if (NGlists == null) NGlists = root.getVectorOfAllLists();
    System.out.println("Number of Lists:" + this.size());
    ByteBuffer rest = ByteBuffer.allocate(this.size() * 12);
    for (int i = 0; i < NGlists.size(); i++) {
      rest.put(((List) NGlists.get(i)).toByteBuffer());
    }
    rest.flip();
    return rest;
  }

  /**
   * @return
   */
  public int size() {
    if (NGlists == null) NGlists = root.getVectorOfAllLists();
    return NGlists.size();
  }

  /**
   * @param data
   */
  public void take(ArcAudioFileDecorator data) {
    root.process(data);
  }

  /**
   * @param b
   */
  public void sort(boolean b) {
    root.sort(b);
  }

  /**
   * @return
   */
  public int getSearchListId() {
    // TODO v1.0 find the Songs list and get it's number would be easy by asking
    // root for Songslist but this is only neccercary if the Gmini220 really
    // needs this
    return 1;
  }

  /**
   * @return
   * 
   */
  public ListAsRoot getRootList() {
    return root;
  }

}