/*
 * $Id: List.java,v 1.2 2005/03/02 08:10:03 gruni Exp $
 * $Author: gruni $ 
 * $Revision: 1.2 $
 * $Date: 2005/03/02 08:10:03 $
 * $Log: List.java,v $
 * Revision 1.2  2005/03/02 08:10:03  gruni
 * structural refactoring made some stuff just better no new functions
 *
 * Revision 1.1  2005/02/27 01:42:36  gruni
 * The New Structure of Lists
 *
 */
package org.jd3lib.archoslib.lists;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Collections;
import java.util.Vector;

import org.jd3lib.archoslib.ArcAudioFileDecorator;
import org.jd3lib.archoslib.Strings;

/**
 * @author Andreas Grunewald
 * 
 * TODO v0.6 Document
 */
public abstract class List implements ArcEntry {

    //LIST TYPES

    public static final byte LIST_ROOT = 0;

    public static final byte LIST_ARTIST = 1;

    public static final byte LIST_ALBUM = 2;

    public static final byte LIST_SONG = 3;

    public static final byte LIST_M3U = 4;

    public static final byte LIST_GENRE = 5;

    public static final byte LIST_YEAR = 6;

    private final byte type; //uint8_t type; list type

    private byte[] offset = null;//new byte[3];//uint24_t offset; pointer to
                                 // list entries

    //private int namePointer = -10;//string_t name; name of list

    /**
     *  
     */
    private short number = -10;

    //short length;//uint16_t length; number of list entries

    private List parent;//file_list_t parent: pointer to parent list

    private Vector entries;

    private final String listName;

    private Strings strs;

    protected List(String defName, byte defType, List parentList) {
        listName = defName;
        type = defType;
        parent = parentList;
        entries = new Vector();
    }

    /**
     * Method which hast to be implemented as Strategy which object are
     * accepted.
     * 
     * @param e
     * @return
     */
    protected abstract boolean accept(ArcEntry e);

    protected void add(ArcEntry e) {
        if (accept(e)) entries.add(e);
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(Object o) {
        if (o instanceof List) {
            return this.getListName().compareToIgnoreCase(
                    ((List) o).getListName());
        }
        throw new ClassCastException("Can not compare these to types");
    }

    //TODO v0.2 implement toByteBuffer
    /*
     * public ByteBuffer toByteBuffer() { ByteBuffer out =
     * ByteBuffer.allocate(12); out.order(ByteOrder.LITTLE_ENDIAN);
     * out.put(type); //uint8_t type; list type out.put(offset);//uint24_t
     * offset; pointer to list entries out.putShort(getLength());//uint16_t
     * length; number of list entries out.putShort(getParent());//file_list_t
     * parent: pointer to parent list out.putInt(namePointer);//string_t name;
     * name of list out.flip(); return out; }
     */

    public Vector getEntries() {
        return entries;
    }

    private String getListName() {
        return listName;
    }

    /**
     * Processes a new File, and adds it to all sublists it fits, for this, the
     * implementing class has to make sure that it calls process everywhere it
     * wants the file to be in to. process can call the add method if it's
     * neccecary to add new sublists to the current list.
     * 
     * @param data
     */
    public abstract void process(ArcAudioFileDecorator data);

    /**
     * @param b
     */
    public void sort(boolean b) {
        Collections.sort(entries);
        if (!b) Collections.reverse(entries);

    }

    public ByteBuffer toByteBuffer() {
        //Check if all preconditions are set.
        if (!(number >= 0 && offset != null && strs != null && getNamePointer() >= 0))
                throw new RuntimeException(
                        "Error in Datacan not create ByteBuffer");
        ByteBuffer out = ByteBuffer.allocate(12);
        out.order(ByteOrder.LITTLE_ENDIAN);
        out.put(type); //uint8_t type; list type
        out.put(offset);//uint24_t offset; pointer to list entries
        out.putShort((short) entries.size());//uint16_t length; number of list
                                             // entries
        out.putShort(getParentNumber());//file_list_t parent: pointer to parent
                                        // list
        out.putInt(getNamePointer());//string_t name; name of list
        out.flip();
        return out;
    }

    /**
     * @return
     */
    private int getNamePointer() {
        return strs.get(getListName());
    }

    /**
     * This method returns the number of the parent Class. It's a default
     * implementation and can be overwritten
     * 
     * @return the parent Number
     */
    protected short getParentNumber() {
        return getParent().getNumber();
    }

    /**
     * @return
     */
    protected List getParent() {
        return parent;
    }

    public String toString() {
        return "(" + getListName() + "):" + entries.toString();
    }

    /**
     * @return the Number of this list;
     */
    public short getNumber() {
        return number;
    }

    /**
     * Sets the parent if parent wasn't set yet. Otherwise it just writes an
     * Error to System.err
     * 
     * @param root
     */
    protected void setParent(List parentList) {
        if (parent == null)
            parent = parentList;
        else
            System.err.println("Parent already set can't set parent twice");

    }

    /**
     * @param s
     */
    public void setNumber(short s) {
        number = s;
    }

    /**
     * @param theStringResolver
     */
    protected void setResolver(Strings theStringResolver) {
        strs = theStringResolver;

    }

    /**
     * @param i
     */
    public void setOffset(int i) {
        //i = i*2;//because it counts as offset in bytes.
        //TODO v2.0 make sure that int is not bigger than 24bit
        offset = new byte[3];
        offset[2] = (byte) ((i << 8) >>> 24);
        offset[1] = (byte) ((i << 16) >>> 24);
        offset[0] = (byte) ((i << 24) >>> 24);

    }

}